/***

showme.c -- a sample program which displays itself

***/

#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>

#include <ehmemtrk.h>        /* memory usage tracking */
#include <ehfiltrk.h>        /* file usage tracking */

/***
This function reads a file into memory,
then prints it to stdout 
***/

void
showme(char *pcFileName)
{
   /*** This is section 1:  initialization ***/

   FILE *volatile pFile = NULL; /* initialize resource handles */
   char *volatile pcBuf = NULL;

   /*** This is Section 2:  main function logic ***/

   EHCATCH(showme) {           

      struct stat Stat;           /* Section 2 has its own scope */
      int iStatResult;            /* local variables go here */

      /* get the file size */
      iStatResult = stat(pcFileName, &Stat);
      if (iStatResult == -1)           /* if an error occurs */
      {
         puts("showme: could not stat the file");
         break;                        /* no need to continue */
      }

      /* allocate a buffer */
      pcBuf = new char[Stat.st_size + 1];
      if (pcBuf == NULL)               /* if an error occurs */
      {
         puts("showme: could not allocate memory");
         break;                        /* no need to continue */
      }

      /* before opening let's make sure the file is writable */
      if (!(Stat.st_mode & S_IWUSR))   /* file not writable by user */
      {
         puts("showme: file not writable");
         break;                        /* no need to continue */
      }

      /* open the file */
      pFile = fopen(pcFileName, "r+"); /* open for update */
      if (pFile == NULL)               /* if an error occurs */
      {
         puts("showme: could not open the file");
         break;                        /* no need to continue */
      }

      /* read it into the buffer */
      if (fread(pcBuf, Stat.st_size, 1, pFile) < 0)
      {
         puts("showme: could not read the file");
         break;
      }

      pcBuf[Stat.st_size] = 0;          /* null-terminate the buffer */

      puts(pcBuf);                      /* print out the file */

   } EHENDCATCH;

   /*** This is Section 3:  clean up and recovery ***/

   if (pFile)                  /* if resource was obtained */
      fclose(pFile);           /* clean up */

   if (pcBuf)                  /* if resource was obtained */
      delete [] pcBuf;         /* clean up */

   EHLEAVE();                  /* track line number of function exit */
}

main(int argc, char **argv)
{
   EHMAIN(argc, argv) {

      showme("showme.c");

   } EHENDMAIN;

   EHRETURN(0);
}
