/***

throwme.c -- a sample program which displays itself,
             throws exceptions on error

***/

#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>

#include <ehmemtrk.h>        /* memory usage tracking */
#include <ehfiltrk.h>        /* file usage tracking */
#include <eherrs.h>          /* standard error codes (e.g., EH_FOPEN) */
#include "tmerrs.h"          /* ThrowMe error codes (see tmerrs.src */

/***
This function reads a file into memory,
then prints it to stdout 
***/

static void throwme(char *pcFileName)
{
   /*** This is section 1:  initialization ***/

   FILE *volatile pFile = NULL; /* initialize resource handles */
   char *volatile pcBuf = NULL;

   /*** This is Section 2:  main function logic ***/

   EHCATCH(throwme) {           

      struct stat Stat;           /* Section 2 has its own scope */
      int iStatResult;            /* local variables go here */

      /* get the file size */
      iStatResult = stat(pcFileName, &Stat);
      if (iStatResult == -1) EHTHROW(EH_STAT);

      /* allocate a buffer */
      pcBuf = new char[Stat.st_size + 1];
      if (pcBuf == NULL) EHTHROW(EH_NEW);

      /* before opening let's make sure the file is writable */
      if (!(Stat.st_mode & S_IWUSR)) EHTHROW(TM_NOTWRITABLE);

      /* open the file */
      pFile = fopen(pcFileName, "r+"); /* open for update */
      if (pFile == NULL) EHTHROW(EH_FOPEN);

      /* read it into the buffer */
      if (fread(pcBuf, Stat.st_size, 1, pFile) < 0) EHTHROW(EH_READ);

      pcBuf[Stat.st_size] = 0;          /* null-terminate the buffer */

      puts(pcBuf);                      /* print out the file */

   } EHENDCATCH;

   /*** This is Section 3:  clean up and recovery ***/

   EHERR("We are in the clean up section.");

   if (pFile)                  /* if resource was obtained */
      fclose(pFile);           /* clean up */

   if (pcBuf)                  /* if resource was obtained */
      delete [] pcBuf;         /* clean up */

   EHLEAVE();                  /* track line number of function exit */
}

static void tracer(char *pcFilename)
{
   EHTRACE(tracer);      // tracer function
   throwme(pcFilename);  // call a catcher function
}

main(int argc, char **argv)
{
   EHMAIN(argc, argv) {

      tracer("throwme.c");

   } EHENDMAIN;

   EHERR("We are in the clean up section.");

   EHRETURN(ehError());        /* let OS know if there was an error */
}
