/***

lrinit.c -- Local Resources, initialization

***/

#include "lrinit.h"   /* "local resource" initialization header */

/***
resource tracking structure
***/

#include <eh.h>       /* error handler definitions */

EHRESTRK lrResTrk;    /* "local resource" resource-tracking structure */


/***
termination function 
***/

static
void
lrTerm() /* local resource termination function */
{
   EHTRACE(lrTerm);      /* function tracing */
   ehResTerm(&lrResTrk); /* de-initialize internal resource tracking */
   EHLEAVE();            /* optional for C++, required for C */
} /* lrTerm */


/***
initialization function
***/

static
void
lrInit() /* local resource initialization function */
{
   EHTRACE(lrInit);

   /* first set up the termination routine */
   ehAtExit(lrTerm);  /* called at EH termination time */

   /* now initialize local resource tracking */
   ehResInit(&lrResTrk);

   EHLEAVE();
} /* lrInit */


/***
external initialization vector
***/

/* pre-empt the Error Handler's external initialization vector */
void (*pfvEhExtInit)() = lrInit;


