/***

t_fildes.c -- test driver for file usage tracking routines

***/

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <eh.h>
#include "lrfildes.h"

static void creat_and_close();
static void creat_and_no_close();
static void open_and_close();
static void open_and_no_close();
static void open_dup_and_close();
static void open_dup_and_no_close();
static void open_dup2_and_close();
static void open_dup2_and_no_close();
static void dup2_stdin_and_close();
static void dup2_stdin_and_no_close();

int
main(int argc, char**argv)
{
   EHMAIN(argc, argv)
   {

      creat_and_close();
      creat_and_no_close();

      open_and_close();
      open_and_no_close();

      open_dup_and_close();
      open_dup_and_no_close();

      open_dup2_and_close();
      open_dup2_and_no_close();

      dup2_stdin_and_close();
      dup2_stdin_and_no_close();

   } EHENDMAIN;

   EHRETURN(ehError());
}


static
void
creat_and_close()
{
   volatile int iFilDes = -1;

   EHCATCH(creat_and_close) {

      iFilDes = creat("t_fildes.1", S_IRWXU);

   } EHENDCATCH;

   if (iFilDes != -1) close(iFilDes);
}


static
void
creat_and_no_close()
{
   volatile int iFilDes = -1;

   EHTRACE(creat_and_no_close);

   iFilDes = creat("t_fildes.2", S_IRWXU);
}

static
void
open_and_close()
{
   volatile int iFilDes = -1;

   EHCATCH(open_and_close) {

      iFilDes = open("t_fildes.1", O_RDONLY);

   } EHENDCATCH;

   if (iFilDes != -1) close(iFilDes);
}


static
void
open_and_no_close()
{
   volatile int iFilDes = -1;

   EHTRACE(open_and_no_close);

   iFilDes = open("t_fildes.1", O_RDONLY);
}

static
void
open_dup_and_close()
{
   volatile int iFilDes1 = -1;
   volatile int iFilDes2 = -1;

   EHCATCH(open_dup_and_close) {

      iFilDes1 = open("t_fildes.1", O_RDONLY);
      iFilDes2 = dup(iFilDes1);

   } EHENDCATCH;

   if (iFilDes2 != -1) close(iFilDes2);
   if (iFilDes1 != -1) close(iFilDes1);
}


static
void
open_dup_and_no_close()
{
   volatile int iFilDes1 = -1;
   volatile int iFilDes2 = -1;

   EHTRACE(open_dup_and_no_close);

   iFilDes1 = open("t_fildes.1", O_RDONLY);
   iFilDes2 = dup(iFilDes1);
}


static
void
open_dup2_and_close()
{
   volatile int iFilDes1 = -1;
   volatile int iFilDes2 = -1;
   volatile int iFilDes3 = -1;

   EHCATCH(open_dup2_and_close) {

      iFilDes1 = open("t_fildes.1", O_RDONLY);
      iFilDes2 = dup(iFilDes1);
      iFilDes3 = dup2(iFilDes1, iFilDes2);

   } EHENDCATCH;

   if (iFilDes3 != -1) close(iFilDes3);
   if (iFilDes2 != -1) close(iFilDes2);
   if (iFilDes1 != -1) close(iFilDes1);
}


static
void
open_dup2_and_no_close()
{
   volatile int iFilDes1 = -1;
   volatile int iFilDes2 = -1;
   volatile int iFilDes3 = -1;

   EHTRACE(open_dup2_and_no_close);

   iFilDes1 = open("t_fildes.1", O_RDONLY);
   iFilDes2 = dup(iFilDes1);
   iFilDes3 = dup2(iFilDes1, iFilDes2);
}


static
void
dup2_stdin_and_close()
{
   volatile int iFilDes1 = -1;

   EHCATCH(dup2_stdin_and_close) {

      iFilDes1 = open("t_fildes.1", O_RDONLY);
      iFilDes1 = dup2(0, iFilDes1);

   } EHENDCATCH;

   if (iFilDes1 != -1) close(iFilDes1);
}


static
void
dup2_stdin_and_no_close()
{
   volatile int iFilDes = -1;

   EHTRACE(dup2_stdin_and_no_close);

   iFilDes = open("t_fildes.1", O_RDONLY);
   iFilDes = dup2(0, iFilDes);
}


