/***

ehbounds.c -- Error Handler, bounds checking support

Copyright (c) 1992-5 Christopher Cobb, All rights reserved.

***/

static const char *rcsid =
       "$Id: ehbounds.c,v 1.6 1996/05/12 15:48:52 ccobb Exp $";

#include "eh.h"

EHBEGINEXTERNC
#include <time.h>
EHENDEXTERNC

#include "ehbounds.h"

FILENAME("ehbounds.c");

/***
mash a value into the desired pattern
***/

static
char
mash A1(int, iVal)
{
   char cVal = (char)(0x8 | (0x3 & (iVal)));
   return (char)( (cVal * 16) + cVal );
}

/***
ehBndSet()
Initialize the boundary region.
***/

void
ehBndSet A2(void *, pvMem,
            size_t, szMem)
{
   char c = mash(time(NULL)); /* seed: 0x88,0x99,0xAA,0xBB */
   char *pc = (char *)pvMem;
   size_t sz = szMem;

   // puts("ehSetBnd: entering");

   //fprintf(EHSTDERR, "ehBndSet: intializing %d bytes at 0x%p\n",szMem,pvMem);

   while (sz--)
      *pc++ = mash(c++);

   //fprintf(EHSTDERR, "ehBndSet: contents of new bounds block:\n");
   //ehHexDmp(EHSTDERR, (char*)pvMem, (int)szMem);
   //fflush(EHSTDERR);

   // puts("ehSetBnd: leaving");
}


/***
ehBndChk()
Check the boundary region.
***/

int /* 0=OK */
ehBndChk A2(void *, pvMem,
            size_t, szMem)
{
   char *pc = (char *)pvMem;
   char c = *pc;

   // puts("ehChkBnd: entering");

   while (szMem--)
   {
      if (*pc++ != mash(c++))
         return 1;
   }

   // puts("ehChkBnd: leaving");

   return 0;
}

