/***

ehgtargs.c -- Error Handler, get command line arguments

Copyright (c) 1992-6 Christopher Cobb, All rights reserved.

***/

static const char*rcsid =
       "$Id: ehgtargs.c,v 1.5 1995/12/04 21:46:15 ccobb Exp $";

/*** system headers ***/

#include "ehenv.h"

EHBEGINEXTERNC

#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <ctype.h>
#include <stdlib.h>

#if ehAMIGA || defined(HAVE_FCNTL_H)
#include <fcntl.h>  /* for isatty() */
#endif

#if ehBC
#include <io.h>     /* for isatty() */
#endif

#ifdef HAVE_UNISTD_H
#include <unistd.h> /* for isatty() */
#endif

EHENDEXTERNC

/*** portability headers ***/
#include "eh.h"
#include "eherrs.h"

FILENAME("ehgtargs.c");

/*** project headers ***/

/*** local (subsystem) headers ***/
#include "ehwldcrd.h"
#include "ehvolume.h"
#include "ehgtargs.h"

/*** file scope (PRIVATE) Macro definitions ***/
#define ISSWITCH(c) ((c)=='-')

/*** file scope (PRIVATE) structure and union typedefs ***/

/*** file scope (PRIVATE) variable definitions ***/

/*** file scope (PRIVATE) function declarations ***/
static void usageHeader A((va_list ap));
static void usageOptions A((va_list ap));
static void chkMsgs A((va_list ap));
static void chkMand A((va_list ap));
static int  nxtArgs A((volatile va_list*, EHGATYPE*, char*, void**, char**));

/***

ehGtArgs()
Get arguments from the command line.
Argc and argv get munged: 
   all the switches a.DHheir parameters get extracted,
   and argc and argv are set to whatever is left over.
The new argc is retuned from this function.
Example:
   ehGtArgs(&argc, &argv,
                   EH_GA_MESSAGE, "Welcome to my program!", 
                   / * list the following according to frequency of use * /
                   EH_GA_IBOOL, 'x', &iArgSolo,         "binary switch",
                   EH_GA_INT,   'i', &iArgValue,        "expects int value",
                   EH_GA_LONG,  'l', &lArgValue,        "expects long value",
                   EH_GA_CHAR,  'z', &cArgValue,        "expects char",
                   EH_GA_MAND|EH_GA_STR, 'q', acArgBuf, "expects string value",
                   0)
***/

void
ehGtArgs A2V(volatile int *, piArgc, 
             char***,        pppcArgv, 
             ...)
{
   volatile int iArgc = *piArgc;
   char   **ppcArgv = *pppcArgv;
   char   **volatile ppcFileV = ppcArgv;  /* pointer to output args */
   volatile va_list ap;
   volatile int iFileCount = 1; /* always at least argv[0] */
   va_list  apSav;
   char     cVarArgSw;
   EHGATYPE GaType;
   void    *pvValue;
   char    *pcDesc;
   char     cCmdLinSw;
   char    *pc;
   char    *pcEndLoc;   /* 1st invalid char found by strtol() */

   EHCATCH(ehGtArgs) {

      /*** initialize stuff ***/

      /* scan incomming parameter list for messages to print */
      VA_START(ap, pppcArgv); 
      chkMsgs(ap);
      va_end(ap);

      /* for Amiga compatability */
      if (iArgc == 2 && strcmp(ppcArgv[1], "?") == 0) EHTHROW(EH_USAGE);

      for (++ppcArgv; --iArgc > 0; ) /* skip over argv[0], loop thru args */
      {
         pc = *ppcArgv++;   /* get next argument from command line */

         /* process Error Handler flags */
         if (memcmp(pc, "EHVOLUME=", 9) == 0)
         {
            ehVolume(pc + 9);
            continue;
         }

         /* if this is NOT a switch, stuff it into argv */
         if (!(ISSWITCH(*pc)) || pc[1] == 0)   /* if '-' or 1-char arg */
         {
            *++ppcFileV = pc; /* muck with the argv array here */
            ++iFileCount;     /* count how many non-switch items are left over */
            continue;
         }

         /* we have a switch: starts with '-' and more than one character long*/
         EHDBG1("switch = [%s]",pc);
         while (*++pc) /* scan cmd line characters in this arg/switch */
         {
            cCmdLinSw = (char)*pc; /* char after '-' */

            /* parse thru switch list */
            VA_START(ap, pppcArgv); 
            for (apSav = ap; 
                   nxtArgs(&ap, &GaType, &cVarArgSw, &pvValue, &pcDesc); 
                   apSav = ap)
            {
               GaType = (EHGATYPE)(GaType & ~GA_MASK); /* strip off GA_MASK bits */
               EHDBG1("checking GaType = [%d]", GaType);
               if (cCmdLinSw == cVarArgSw) /* match cmd line w lst of switches*/
               {
                  *(EHGATYPE*)apSav = (EHGATYPE)(*(EHGATYPE*)apSav | GA_PRES); /* mark VarArgSw as present */
                  EHDBG("GaType matches command line arg");
                  break; /* a char on the command line matches a switch let */
               }
            } /* end for */
            va_end(ap);

            if (GaType == 0)
            {
               fprintf(EHSTDOUT, "ERROR: don't recognize switch -%c\n", cCmdLinSw);
               EHTHROW(EH_USAGE);
            }

            EHDBG1("outer switch(GaType=[%d])", GaType);
            switch (GaType) /* outer switch */
            {
            case EH_GA_BOOL:  *(bool *)pvValue = true; continue;
            case EH_GA_CBOOL: *(char *)pvValue = '\1'; continue;
            case EH_GA_HBOOL: *(short *)pvValue = 1;   continue;
            case EH_GA_IBOOL: *(int *)pvValue  = 1;    continue;

            default: /* supplied value either follows immediately or is nxt arg */
               if (*++pc == 0) /* must be next arg */
               {
                  if (--iArgc <= 0)
                  {
                     fprintf(EHSTDOUT, "ERROR: switch -%c needs argument: %s\n\n",
                           cCmdLinSw, pcDesc);
                     EHTHROW(EH_USAGE);
                  }
                  pc = *ppcArgv++; /* let's look at next arg */
                  if ((ISSWITCH(*pc)) && pc[1] != 0)   /* a switch was given! */
                  {
                     fprintf(EHSTDOUT, "ERROR: switch -%c needs argument\n\n",
                           cCmdLinSw, pcDesc);
                     EHTHROW(EH_USAGE);
                  }

               }

               EHDBG1("inner switch(GaType=[%d])", GaType);
               switch (GaType) /* inner switch */
               {
               case EH_GA_CHAR: *(char *)pvValue = *pc; break;
               case EH_GA_ICHAR: *(int *)pvValue = (unsigned char)*pc; break;
               case EH_GA_PCHAR: *(char **)pvValue = pc; break;
               case EH_GA_ACHAR: strcpy((char*)pvValue, pc); break;

               case EH_GA_SHORT:
               case EH_GA_INT:
               case EH_GA_SIZE_T:
               case EH_GA_LONG:
               case EH_GA_FLOAT:
               case EH_GA_DOUBLE:
                  if (strchr("+-0123456789.eE", (int)*pc) == NULL)
                  {
                     fprintf(EHSTDOUT,
                           "ERROR: switch -%c needs a numeric argument\n", 
                           cCmdLinSw);
                     EHTHROW(EH_USAGE);
                  }
                  if (GaType == EH_GA_SHORT)
                     *(short *)pvValue = (short)strtol(pc, &pcEndLoc, 0);
                  else if (GaType == EH_GA_INT)
                     *(int *)pvValue = (int)strtol(pc, &pcEndLoc, 0);
                  else if (GaType == EH_GA_SIZE_T)
                     *(size_t *)pvValue = (size_t)strtol(pc, &pcEndLoc, 0);
                  else if (GaType == EH_GA_LONG)
                     *(long *)pvValue = (long)strtol(pc, &pcEndLoc, 0);
                  else if (GaType == EH_GA_FLOAT)
                     *(float *)pvValue = (float)strtod(pc, &pcEndLoc);
                  else
                     *(double *)pvValue = (double)strtod(pc, &pcEndLoc);
                  break;

               default:
                  EHERR1("GaType = [%d]", GaType);
                  EHTHROW(EH_SWITCH); /* bad value passed to switch */
               } /* end inner switch */
               pc += strlen(pc) - 1;
               
            } /* end outer switch */
         } /* end while */
      } /* for (all cmd line args) */

      VA_START(ap, pppcArgv); 
      chkMand(ap);
      va_end(ap);

   } EHENDCATCH;

   if (ehError())
   {
      VA_START(ap, pppcArgv); 
      usageHeader(ap);
      va_end(ap);
      VA_START(ap, pppcArgv); 
      usageOptions(ap);
      va_end(ap);
   }
   else
   {
      /* tidy up original argc/argv */
      *piArgc = iFileCount;
      *++ppcFileV = NULL;   /* null term arg list */

      /* now expand wild cards in file-only list */
      ehExpWildCards((int*)piArgc, pppcArgv);
   }

   EHLEAVE();
} /* ehGtArgs */


/***

usageHeader(FILE*, ap)
Check for and print out usage text.

***/

static
void
usageHeader A1(va_list, ap)
{
   EHGATYPE GaType; 
   char   cVarArgSw;
   void  *pvValue;
   char  *pcDesc;

   EHTRACE(usageHeader);

   fprintf(EHSTDOUT, "usage: %s ", Eh.pcProgName);
   while (nxtArgs(&ap, &GaType, &cVarArgSw, &pvValue, &pcDesc))
   {
      if (GaType == EH_GA_USAGE)
      {
         fprintf(EHSTDOUT, "%s", pcDesc);
         break;
      }
   }
   if (GaType != EH_GA_USAGE)
   {
      fprintf(EHSTDOUT, "[ options ]");
   }
   fprintf(EHSTDOUT, "\n");

   EHLEAVE();
} /* usageHeader() */



/***

usage(ap)
If the user has provided a EH_GA_USAGE string use it.
Otherwise, print out generic usage message.

***/

static
void 
usageOptions A1(va_list, ap)
{
   EHGATYPE  GaType; 
   char    cVarArgSw;
   void   *pvValue;
   char   *pcDesc;

   EHTRACE(usageOptions);

   while (nxtArgs(&ap, &GaType, &cVarArgSw, &pvValue, &pcDesc))
   {
      char *pc;
      char  acBuf[64];

      if ( (GaType & ~GA_MASK) == EH_GA_MESSAGE ) continue;
      if ( (GaType & ~GA_MASK) == EH_GA_USAGE ) continue;

      pc = acBuf;

      if ((GaType & EH_GA_MAND) == 0) /* NOT mandatory */
      {
         strcpy(pc, "[ ");
         while (*++pc)
            ;
      }

      switch (GaType & ~GA_MASK)
      {
      case EH_GA_BOOL :
      case EH_GA_CBOOL :
      case EH_GA_HBOOL :
      case EH_GA_IBOOL :
         sprintf(pc, "-%c ", cVarArgSw);
         while (*++pc)
            ;
         break;

      case EH_GA_SHORT :
      case EH_GA_INT :
      case EH_GA_SIZE_T :
      case EH_GA_LONG :
      case EH_GA_FLOAT :
      case EH_GA_DOUBLE :
         sprintf(pc, "-%c<number> ", cVarArgSw);
         while (*++pc)
            ;
         break;

      case EH_GA_CHAR :
      case EH_GA_ICHAR :
         sprintf(pc, "-%c<char> ", cVarArgSw);
         while (*++pc)
            ;
         break;

      case EH_GA_PCHAR :
      case EH_GA_ACHAR :
         sprintf(pc, "-%c<string> ", cVarArgSw);
         while (*++pc)
            ;
         break;

      } /* end switch */

      if ((GaType & EH_GA_MAND) == 0)
      {
         strcpy(pc, "] ");
         while (*++pc)
            ;
      }

      fprintf(EHSTDOUT, "          %-16.16s     %s\n", acBuf, pcDesc);
   }

   fprintf(EHSTDOUT, "\n");
   EHLEAVE();
} /* usageOptions */


/***

chk'(ap)
Check for and print out any messages from the caller.

***/

static
void
chkMsgs A1(va_list, ap)
{
   EHGATYPE GaType; 
   char   cVarArgSw;
   void  *pvValue;
   char  *pcDesc;

   EHTRACE(chkMsgs);

   while (nxtArgs(&ap, &GaType, &cVarArgSw, &pvValue, &pcDesc))
   {
      if (GaType == EH_GA_MESSAGE)
      {
         /* use stderr so not to interfere with normal output */
         fprintf(EHSTDOUT, "%s\n", pcDesc);
      }
   }

   EHLEAVE();
} /* chkMsgs() */



/***

chkMand(ap)
Check that mandatory arguments have been provided.

***/

static
void
chkMand A1(va_list, ap)
{
   EHGATYPE GaType; 
   char   cVarArgSw;
   void  *pvValue;
   char  *pcDesc;
   int    iNeedUsage = 0;

   EHTRACE(chkMand);

   while (nxtArgs(&ap, &GaType, &cVarArgSw, &pvValue, &pcDesc))
   {
      if ( (GaType & EH_GA_MAND) != 0 && (GaType & GA_PRES) == 0 )
      {
         fprintf(EHSTDOUT, "ERROR: switch -%c is mandatory: %s\n", cVarArgSw, pcDesc);
         iNeedUsage = 1;
      }
   } /* end for */
   if (iNeedUsage)
   {
      fprintf(EHSTDOUT, "\n");
      EHTHROW_LEAVE(EH_USAGE); /* this forces usage statement */
   }
   EHLEAVE();
} /* chkMand() */


/***

nxtArgs(ap, EHGATYPE *, char *, void *, char *)
Get the next set of arguments from the variable argument list.

***/

static
int
nxtArgs A5(volatile va_list*, pVaList,    /* argument pointer */
           EHGATYPE *,        pGaType,    /* where to put EHGATYPE */
           char *,            pcVarArgSw, /* where to put nxt var arg char */
           void **,           ppvValue,   /* where to put addr of target */
           char **,           ppcDesc)    /* where to put address of text */
{
   volatile int iRetVal = 1; /* assume we're not done */
   int iGaType;

   EHTRACE(nxtArgs);

   EHDBG1("iRetVal = [%d]", iRetVal);
   *pGaType = (EHGATYPE)va_arg(*pVaList, int);
   EHDBG1("picked off *pGaType = [%d]", *pGaType & ~GA_MASK);
   switch (iGaType = (*pGaType & ~GA_MASK))
   {
   case 0:
      iRetVal = 0; /* means we're done */
      break;

   case EH_GA_MESSAGE:
   case EH_GA_USAGE:
      *pcVarArgSw = NUL; /* these aren't used w EH_GA_MESSAGE or EH_GA_USAGE */
      *ppvValue = NULL;
      break;

   case EH_GA_BOOL:
   case EH_GA_CBOOL:
   case EH_GA_HBOOL:
   case EH_GA_IBOOL:
   case EH_GA_CHAR:
   case EH_GA_ICHAR:
   case EH_GA_PCHAR:
   case EH_GA_ACHAR:
   case EH_GA_SHORT:
   case EH_GA_INT:
   case EH_GA_SIZE_T:
   case EH_GA_LONG:
   case EH_GA_FLOAT:
   case EH_GA_DOUBLE:
      *pcVarArgSw = (char)va_arg(*pVaList, int);
      EHDBG1("picked off *pcVarArgSw = [%c]", *pcVarArgSw);
      *ppvValue = va_arg(*pVaList, void *);
      break;

   default:
      EHTHROW_RETURN(EH_SWITCH, iRetVal); /* bad value passed to switch */
   }
   if (iGaType)
   {
      *ppcDesc = (char *)va_arg(*pVaList, char *);
      EHDBG1("picked off *ppcDesc = [%s]", *ppcDesc);
   }

   EHDBG1("returning [%d]", iRetVal);
   EHRETURN(iRetVal);
} /* nxtArgs() */



