/***

ehintini.c -- Error Handler, internal initialization

Copyright (c) 1992-6 Christopher Cobb, All rights reserved.

***/

static const char *rcsid =
       "$Id: ehintini.c,v 1.13 1996/05/12 15:48:18 ccobb Exp $";

#include "ehenv.h"

#if ehAMIGA
#include "ehamiga.h"
#endif /* ehAMIGA */

#include "eh.h"
#include "eherrs.h"
#include "ehintres.h"
#include "ehintini.h"

/***
   Slash used in path names.
***/
const char *pcEhSlash =
#if ehMSDOS
   "\\";
#else
   "/";
#endif

/***

This function contains any internal (os-specific)
initialization that takes place in PGM_ENTER()
after ehPgmEnt() and ehFunEnt() are called.
This is therefore written as a full/normal eh-compliant
function.

This function is called via the function pointer
pfvEhIntInit to replicate the same structure that
is needed for external initialization.

***/

void
ehIntInit A0()
{
   // puts("ehIntInit: about to call FUN_TRACE(ehIntInit)");

   FUN_TRACE("ehIntInit");

   // puts("ehIntInit: back from FUN_TRACE(ehIntInit)");

/***
   the following conditional compilation(s)
   represent code needed to properly set up
   the STANARD C environment.  This may or may
   not be the same as code needed to set up a
   non-standard C environment.  Access to 
   environment-specific features is sometimes
   needed to properly setup operation of the
   Error Handler.
***/

#if ehAMIGA

   /* first set up the recovery routine -- in case of a failure */
   ehAtExit(ehAmResetExceptCode);  /* at term time */

   /* now set up control-c handling code */
   ehAmSetExceptCode(ehAmSigInt, (ULONG)SIGBREAKF_CTRL_C, (void*)NULL);

#endif

   /* first set up the termination routine -- in case of a failure */
   // puts("ehIntInit: about to call ehAtExit(ehIntResTerm)");
   ehAtExit(ehIntResTerm);  /* at term time */

   /* now initialize internal resource tracking */
   // puts("ehIntInit: about to call ehIntResInit()");
   ehIntResInit();

   FUN_LEAVE();
} /* ehIntInit */

