/***

ehlist.c -- Error Handler, linked list manager

Copyright (c) 1992-6 Christopher Cobb, All rights reserved.

***/

static const char*rcsid =
       "$Id: ehlist.c,v 1.5 1996/03/04 06:51:14 ccobb Exp $";

#include "eh.h"
#include "eherrs.h"
#include "ehlist.h"

/* unnamed/non-prioritized list support */

bool
ehAddHead A2(EHLIST *, pEhList,
             EHNODE *, pEhNode)
{
   EHNODE *pEhNodeHead = pEhList->pEhNodeHead;  /* current head of the list */

   //puts("ehAddHead: about to invoke FUN_TRACE(ehAddHead)");
   FUN_TRACE("ehAddHead");

   //puts("ehAddHead: about to call EHASSERT_LEAVE(pEhList != NULL)");
   EHASSERT_RETURN(pEhList != NULL, false);

   //puts("ehAddHead: about to call EHASSERT_LEAVE(pEhNode != NULL)");
   EHASSERT_RETURN(pEhNode != NULL, false);

   //printf("ehAddHead: adding node 0x%p to list 0x%p\n", pEhList, pEhNode);

   /* take care of new node */
   pEhNode->pEhNodeSucc = pEhNodeHead;
   pEhNode->pEhNodePred = (EHNODE *)pEhList;

   /* take care of old head of list */
   pEhNodeHead->pEhNodePred = pEhNode;

   /* take care of list header */
   pEhList->pEhNodeHead = pEhNode;

   //ehCallStack();
   //puts("ehAddHead: about to invoke FUN_LEAVE()");
   FUN_RETURN(true);
} /* ehAddHead */


bool
ehAddTail A2(EHLIST *, pEhList,
             EHNODE *, pEhNode)
{
   EHNODE *pEhNodeTail = pEhList->pEhNodeTailPred;  /* this is the current tail of the list */

   FUN_TRACE("ehAddTail");

   EHASSERT_RETURN(pEhList != NULL, false);
   EHASSERT_RETURN(pEhNode != NULL, false);

   /* take care of new node */
   pEhNode->pEhNodeSucc = (EHNODE *)&(pEhList->pEhNodeTail);
   pEhNode->pEhNodePred = pEhNodeTail;

   /* take care of old tail of list */
   pEhNodeTail->pEhNodeSucc = pEhNode;

   /* take care of list header */
   pEhList->pEhNodeTailPred = pEhNode;

   FUN_RETURN(true); /* function succeeded */
} /* ehAddTail */


void
ehInsert A3(EHLIST *, pEhList,
            EHNODE *, pEhNode,
            EHNODE *, pEhNodePred)
{
   EHNODE *pEhNodeSucc = pEhNodePred->pEhNodeSucc;

   FUN_TRACE("ehInsert");

   EHASSERT_LEAVE(pEhList != NULL);
   EHASSERT_LEAVE(pEhNode != NULL);
   EHASSERT_LEAVE(pEhNodePred != NULL);

   /* take care of new node */
   pEhNode->pEhNodeSucc = pEhNodeSucc;
   pEhNode->pEhNodePred = pEhNodePred;

   /* take care of predecessor */
   pEhNodePred->pEhNodeSucc = pEhNode;

   /* take care of successor */
   pEhNodeSucc->pEhNodePred = pEhNode;

   FUN_LEAVE();
} /* ehInsert */


bool
ehListIsEmpty A1(EHLIST *, pEhList)
{
   FUN_TRACE("ehListIsEmpty");

   EHASSERT_RETURN(pEhList != NULL, false);

   if (pEhList->pEhNodeTailPred == (EHNODE *)pEhList)
   {
      EHASSERT_RETURN(pEhList->pEhNodeHead->pEhNodeSucc == NULL, false);
      FUN_RETURN(true);
   }

   FUN_RETURN(false);
} /* ehListIsEmpty */


void
ehNewList A1(EHLIST *, pEhList)
{
   FUN_TRACE("ehNewList");

   EHASSERT_LEAVE(pEhList != NULL);

   pEhList->pEhNodeHead     = (EHNODE *)&(pEhList->pEhNodeTail);
   pEhList->pEhNodeTail     = 0;
   pEhList->pEhNodeTailPred = (EHNODE *)pEhList;

   FUN_LEAVE();
} /* ehNewList */


EHNODE *
ehRemHead A1(EHLIST *, pEhList)
{
   EHNODE *pEhNodeRem;
   EHNODE *pEhNodeRemSucc;

   FUN_TRACE("ehRemHead");

   EHASSERT_RETURN(pEhList != NULL, (EHNODE*)NULL);
   if (ehListIsEmpty(pEhList))
   {
      //EHDBG("list is empty");
      FUN_RETURN((EHNODE*)NULL);
   }

   pEhNodeRem = pEhList->pEhNodeHead;
   if (pEhNodeRem == NULL) /* uninitialized list ? */
   {
      EHERR("uninitialized list");
      FUN_RETURN((EHNODE*)NULL); /* silent return */
   }

   pEhNodeRemSucc = pEhNodeRem->pEhNodeSucc;

   /* take care of the removed node's successor */
   pEhNodeRemSucc->pEhNodePred = (EHNODE *)pEhList;

   /* take care of the list header */
   pEhList->pEhNodeHead = pEhNodeRemSucc;

   /* take care of the removed node */
   pEhNodeRem->pEhNodeSucc = NULL;
   pEhNodeRem->pEhNodePred = NULL;

   FUN_RETURN(pEhNodeRem);
} /* ehRemHead */

/***
This function is a new addition and is not
available directly in the Amig Exec Library.
It Inquires about the presence of a message
without removing it from the list.
***/
EHNODE *
ehInqHead A1(EHLIST *, pEhList)
{
   EHNODE *pEhNodeInq = pEhList->pEhNodeHead;

   FUN_TRACE("ehInqHead");

   EHASSERT_RETURN(pEhList != NULL, (EHNODE*)NULL);

   if (ehListIsEmpty(pEhList))
      FUN_RETURN((EHNODE*)NULL);

   FUN_RETURN(pEhNodeInq);
} /* ehInqHead */


bool
ehRemove A1(EHNODE *, pEhNode)
{
   EHNODE *pEhNodePred;
   EHNODE *pEhNodeSucc;

   FUN_TRACE("ehRemove");

   //puts("about to call EHASSERT_LEAVE()");
   EHASSERT_RETURN(pEhNode != NULL, false);

   //puts("setting pEhNodePred");
   pEhNodePred = pEhNode->pEhNodePred;
   //puts("setting pEhNodeSucc");
   pEhNodeSucc = pEhNode->pEhNodeSucc;

   /* take care of predecessor */
   EHASSERT_RETURN(pEhNodePred, false);
   EHASSERT_RETURN(pEhNodePred->pEhNodeSucc, false);
   pEhNodePred->pEhNodeSucc = pEhNode->pEhNodeSucc;

   /* take care of successor */
   //puts("setting pEhNodeSucc->pEhNodePred");
   pEhNodeSucc->pEhNodePred = pEhNode->pEhNodePred;

   /* take care of removed node */
   //puts("setting pEhNode->pEhNodeSucc");
   pEhNode->pEhNodeSucc = NULL;
   //puts("setting pEhNode->pEhNodePred");
   pEhNode->pEhNodePred = NULL;

   //puts("about to call FUN_LEAVE()");
   FUN_RETURN(true);
} /* ehRemove */


EHNODE *
ehRemTail A1(EHLIST *, pEhList)
{
   EHNODE *pEhNodeRem = pEhList->pEhNodeTailPred;

   FUN_TRACE("ehRemTail");

   EHASSERT_RETURN(pEhList != NULL, (EHNODE*)NULL);
   EHASSERT_RETURN(!ehListIsEmpty(pEhList), (EHNODE*)NULL);

   /* take care of tail's predecessor */
   pEhNodeRem->pEhNodePred->pEhNodeSucc = (EHNODE *)&(pEhList->pEhNodeTail);

   /* take care of list header */
   pEhList->pEhNodeTailPred = pEhNodeRem->pEhNodePred;

   /* take care of removed node */
   pEhNodeRem->pEhNodeSucc = NULL;
   pEhNodeRem->pEhNodePred = NULL;

   FUN_RETURN(pEhNodeRem);
} /* ehRemTail */



/* named/prioritized list support */


void    
ehEnqueue A2(EHLIST *, pEhList,
             EHNODE *, pEhNode)
{
   FUN_TRACE("ehEnqueue");
   NOREF(pEhList);
   NOREF(pEhNode);
   FUN_LEAVE();
} /* ehEnqueue */


EHNODE *
ehFindName A2(EHLIST *, pEhList,
              byte *,   pbName)
{
   FUN_TRACE("ehFindName");
   NOREF(pEhList);
   NOREF(pbName);
   FUN_RETURN((EHNODE*)NULL);
} /* ehFindName */


/*** additional support functions not found in Amiga Exec ***/

/* locate tail */
EHNODE *
ehLocTail A1(EHLIST *, pEhList)
{
   FUN_TRACE("ehLocTail");

   EHASSERT_RETURN(pEhList != NULL, (EHNODE*)NULL);
   EHASSERT_RETURN(!ehListIsEmpty(pEhList), (EHNODE*)NULL);

   FUN_RETURN(pEhList->pEhNodeTailPred);
} /* ehLocTail */

