/***

ehosterm.c -- Error Handler, OS termination routines

Copyright (c) 1992-6 Christopher Cobb, All rights reserved.

***/

static const char *rcsid =
       "$Id: ehosterm.c,v 1.13 1996/05/12 15:48:18 ccobb Exp $";

#include "eh.h"
#include "eherrs.h"
#include "ehosterm.h"

#define TERMLIST struct _term_list_ /* termination function list */
TERMLIST
{
   pfv_v     pfvTermFunc;  /* termination function */
   TERMLIST *pTermList;    /* next function to call */
};

static TERMLIST *pTermListHeader; /* anchor for TERMLIST linked list */

/***

The function loads up function pointers
which get called at ehOsTerm()...
sort of like atexit().

***/

void
ehAtExit A1(pfv_v, pfvTermFunc)
{
   TERMLIST *pTermList;

   // puts("ehAtExit: about to call FUN_TRACE(ehAtExit)");

   FUN_TRACE("ehAtExit");

   // puts("ehAtExit: back from  FUN_TRACE(ehAtExit)");

   pTermList = NEW(TERMLIST);
   if (pTermList == NULL) EHTHROW_LEAVE(EH_NEWMACRO);

   pTermList->pfvTermFunc = pfvTermFunc; /* function to call at ehOsTerm() time */
   pTermList->pTermList = pTermListHeader; /* attach to list */
   pTermListHeader = pTermList;

   // puts("ehAtExit: about to call FUN_LEAVE()");

   FUN_LEAVE();

   // puts("ehAtExit: back from  FUN_LEAVE()");

} /* ehAtExit */


/***

This function executes functions that were
specified with the ehAtExit() call:
last first

***/

void
ehOsTerm A0()
{
   TERMLIST *pTermList;

   FUN_TRACE("ehOsTerm");

   pTermList = pTermListHeader; /* last one first */
   while (pTermList)
   {
      TERMLIST *pTL = pTermList; /* temp copy */
      (*(pTermList->pfvTermFunc))();
      pTermList = pTermList->pTermList;
      EHFREE(pTL);
   }

#if TRACKING_INTERNAL_MEMORY_USE
   mtChkMem();
#endif

   FUN_LEAVE();
} /* ehOsTerm */

