/***

ehresini.c -- Error Handler, resource tracking initialization

Copyright (c) 1992-6 Christopher Cobb, All rights reserved.

***/

static const char*rcsid =
       "$Id: ehresini.c,v 1.5 1995/12/11 07:26:29 ccobb Exp $";

/*** standard headers ***/

#include "ehenv.h"

EHBEGINEXTERNC
#include <stdlib.h>
#include <string.h>
EHENDEXTERNC

/*** error handler headers ***/

#include "eh.h"
#include "eherrs.h"

/*** subsystem headers ***/

#include "ehrestrk.h"
#include "ehavltre.h"
#include "ehresini.h"

#if     TRACKING_INTERNAL_MEMORY_USE
#include <mt.h>
#endif

FILENAME("ehresini.c");

char cEhAvlDmp = NUL;
char cEhResDmp = NUL;

static int  cmpResList      A((const void *pv1, const void *pv2));
static int  freeResList     A((const void**ppv));
static void resDump         A((EHAVLNODE*pEhAvlNode,int iLevel));
static void resDumpNode     A((const void*pvUserData,int iLevel));
static void resDumpUserData A((FILE *pFile, const void *pvUserData));

static int  cmpFunList      A((const void *pv1, const void *pv2));
static int  freeResFunc     A((const void** ppv));
static void funDumpUserData A((FILE *pFile, const void *pvUserData));

/***
ehResInit()
Initialize resource tracking.
***/

void
ehResInit A1(EHRESTRK*, pEhResTrk)
{
   pfi_pvpv    pfiCmpFun = cmpFunList;
   pfi_ppv     pfiDelFun = freeResFunc;
   pfv_pFilepv pfvDmpFun = funDumpUserData;
   pfi_pvpv    pfiCmpRes = cmpResList;
   pfi_ppv     pfiDelRes = freeResList;
   pfv_pFilepv pfvDmpRes = resDumpUserData;

   // puts("ehResInit: about to call FUN_TRACE(ResIni)");
   FUN_TRACE("ehResInit");
   // puts("ResIni: back from FUN_TRACE(ResIni)");

   // puts("ResIni: about to call ehAvlInit(Fun)");
   pEhResTrk->pEhAvlFun = ehAvlInit(pfiCmpFun, pfiDelFun, pfvDmpFun);

   // puts("ResIni: about to call ehAvlInit(Res)");
   pEhResTrk->pEhAvlRes = ehAvlInit(pfiCmpRes, pfiDelRes, pfvDmpRes);

   // puts("ResIni: about to call ehNewList()");
   ehNewList(&(pEhResTrk->EhList));  /* initialize list header */

   // puts("ResIni: setting isInitialized to true");
   pEhResTrk->isInitialized = true;

   // puts("ResIni: about to call FUN_LEAVE()");
   FUN_LEAVE();
   // puts("ResIni: back from FUN_LEAVE()");
} /* ehResInit() */


/***
cmpResList()
Compare EHRESLIST elements (used by ehInSort and ehRmSort).
This function compares two EHRESLIST elements.  It returns zero if the
ulResHndl members of the two elements are equal, -1 if the first arg is less
than the second, and +1 if the first is greater than the second.

NOTE: This function can be substantially optimized by using:
         iRetVal = lResHndl1 - iResHndl2
      However, this is more difficult to debug.

RETURN VALUES:
-1: first < second
 0: first == second
+1: first > second
***/

static 
int 
cmpResList A2(const void *, pv1,
              const void *, pv2)
{
   long lResHndl1 = (long)((EHRESLIST*)pv1)->ulResHndl;
   long lResHndl2 = (long)((EHRESLIST*)pv2)->ulResHndl;
   volatile int iRetVal = 0;

   EHCHAN(2);
   FUN_TRACE("cmpResList");   /* track entry into this function */
   
#if (EH_CHAN == 0) && 0

   EHDBG2("ulResHndl1 = 0x%lx, ulResHndl2 = 0x%lx",lResHndl1,lResHndl2);

   if (lResHndl1 < lResHndl2)
   {
      iRetVal = -1;
   }
   else if (lResHndl1 > lResHndl2)
   {
      iRetVal = 1;
   }
   else
   {
      long lResType1 = (long)((EHRESLIST*)pv1)->ulFlags & RES_TYPE_MSK;
      long lResType2 = (long)((EHRESLIST*)pv2)->ulFlags & RES_TYPE_MSK;

      if (lResType1 < lResType2)
      {
         iRetVal = -1;
      }
      else if (lResType1 > lResType2)
      {
         iRetVal = 1;
      }
      else
      {
         iRetVal = 0;
      }
   }

#else

   iRetVal = (int)(lResHndl1 - lResHndl2);
   if (iRetVal == 0)
   {
      short hResType1 = (short)((EHRESLIST*)pv1)->iType;
      short hResType2 = (short)((EHRESLIST*)pv2)->iType;

      iRetVal = (int)(hResType1 - hResType2);
   }

#endif

   EHDBG1("returning %i", iRetVal);
   FUN_RETURN(iRetVal);
} /* cmpResList */


/***
freeResList()
Free all components of a RESLIST structure
***/

static
int
freeResList A1(const void**, ppv)
{
   EHRESLIST**ppEhResList = (EHRESLIST**)ppv;

   FUN_TRACE("freeResList");

   EHDBG("checking *ppEhResList");
   if (*ppEhResList)
   {
      EHDBG("checking *ppEhResList->pEhResCall");
      if ((*ppEhResList)->pEhResCall)
      {
         EHDBG("freeing *ppEhResList->pEhResCall");
         EHFREE((*ppEhResList)->pEhResCall);
      }

      EHDBG("freeing *ppEhResList");
      EHFREE(*ppEhResList);
   }

   EHDBG("returning");
   FUN_RETURN(0);
} /* freeResList */


/***
ehResTerm()
Check for any outstanding resources and release those still in use.
***/

void
ehResTerm A1(EHRESTRK *, pEhResTrk)
{
   int iBadRes = 0;
   EHRESCALL *pEhResCall = NULL;

   EHCATCH(ehResTerm) {
   /* no section two */
   } EHENDCATCH;

   if (pEhResTrk->szResCount) /* did we obtain any of this resource ? */
   {
      fprintf(EHSTDERR, "\n");
      fflush(EHSTDERR);
   }

   if (pEhResTrk->ulResPeak != 0 || pEhResTrk->szResCount > 0)
   {
      fprintf(EHSTDERR, "# res slots used = %ld\n",
            (long)pEhResTrk->szResCount);
      fprintf(EHSTDERR, "# res obtained   = %lu\n",
            pEhResTrk->ulResObtained);
      fflush(EHSTDERR);
   }

   while ((pEhResCall =
         (EHRESCALL *)ehRemHead(&(pEhResTrk->EhList))) != NULL)
   {
      EHRESLIST *pRes = pEhResCall->pEhResList; /* pick up parent */

      EHREM1("ulResHndl 0x%lx", pRes->ulResHndl);
      if (pRes->pcResNam) EHREM1("pcResNam %s", pRes->pcResNam);
      EHREM1("EhResState %i", pRes->EhResState);
      EHREM1("szRCAlloc %lu", pRes->szRCAlloc);
      EHREM1("szRCInUse %lu", pRes->szRCInUse);
      if (pRes->ulResHndl &&
            (pRes->EhResState == EH_RES_OBTAINED || pRes->szRCInUse))
      {
         if (pRes->szRCInUse--)
         {
            const char *pcFmt2 = " obtained at <%s %s%s%u> pass %lu";
            int iNumArgs;

            if (pRes->pcResNam)
            {
               const char *pcFmt = "unreleased \"%s\"";

               fprintf(EHSTDERR, pcFmt, pRes->pcResNam);
               if (EHSTDERR != EHSTDOUT)
                     fprintf(EHSTDOUT, pcFmt, pRes->pcResNam);
            }
            else
            {
               const char *pcFmt = "unreleased resource %lu (0x%lx)";

               fprintf(EHSTDERR, pcFmt, pRes->ulResHndl, pRes->ulResHndl);
                  if (EHSTDERR != EHSTDOUT) fprintf(EHSTDOUT, pcFmt,
                                        pRes->ulResHndl, pRes->ulResHndl);
            }

            fprintf(EHSTDERR, pcFmt2,
               pEhResCall->pcFileO, 
               pEhResCall->pcFuncO?pEhResCall->pcFuncO:"",
               pEhResCall->pcFuncO?"() ":"",
               pEhResCall->iLineO,
               pEhResCall->ulCountO);
            if (EHSTDERR != EHSTDOUT)
               fprintf(EHSTDOUT, pcFmt2,
                  pEhResCall->pcFileO,
                  pEhResCall->pcFuncO?pEhResCall->pcFuncO:"",
                  pEhResCall->pcFuncO?"() ":"",
                  pEhResCall->iLineO,
                  pEhResCall->ulCountO);

            fprintf(EHSTDERR, "\n");
            if (EHSTDERR != EHSTDOUT) fprintf(EHSTDOUT, "\n");
            fflush(EHSTDERR);

            /*** handle unreleased resources here ***/

            EHASSERT_LEAVE(pRes->pfvDefRel != NULL);
            iNumArgs = (int)(pRes->ulFlags & EH_RES_NUMARGS_MSK);
            EHREM1("iNumArgs = %d", iNumArgs);
            EHREM1("pfvDefRel = %p", pRes->pfvDefRel);
            EHREM1("free = %p", free);
            switch (iNumArgs)
            {
            default: EHTHROW_LEAVE(EH_SWITCH);     break;
            case 0: (*pRes->pfvDefRel)(0L); break;
            case 1: (*pRes->pfvDefRel)(pEhResCall->pulArgs[0]); break;
            case 2: (*pRes->pfvDefRel)(pEhResCall->pulArgs[0],
                                       pEhResCall->pulArgs[1]); break;
            case 3: (*pRes->pfvDefRel)(pEhResCall->pulArgs[0],
                                       pEhResCall->pulArgs[1],
                                       pEhResCall->pulArgs[2]); break;
            case 4: (*pRes->pfvDefRel)(pEhResCall->pulArgs[0],
                                       pEhResCall->pulArgs[1],
                                       pEhResCall->pulArgs[2],
                                       pEhResCall->pulArgs[3]); break;
            }

            /*** end of special resource handling ***/

            if (pEhResCall->pulArgs) EHFREE(pEhResCall->pulArgs);
            pEhResCall->pcFileO = NULL;
            pEhResCall->pcFuncO = NULL;
         }
         iBadRes = 1;
      } /* end if OBTAINED or InUse */
      if (pRes->pcResNam)
         EHFREE(pRes->pcResNam);
      if ((pRes->szRCInUse == 0) && pRes->pEhResCall)
         EHFREE(pRes->pEhResCall);
   }

   ehAvlTerm(pEhResTrk->pEhAvlRes);
   ehAvlTerm(pEhResTrk->pEhAvlFun);

   if (iBadRes)
   {
      long  lDelta;
      const char *pcMsg =
         "\n*** Resource usage problems: %ld resource%s not released ***\n\n";

      lDelta = (long)(pEhResTrk->ulResObtained - pEhResTrk->ulResReleased);
      fprintf(EHSTDOUT, pcMsg,  lDelta, lDelta == 1 ? "" : "s");
      fflush(EHSTDOUT);
      //printf("ulResObtained is %lu, ulResReleased is %lu\n", 
            //pEhResTrk->ulResObtained, pEhResTrk->ulResReleased);
      EHLEAVE();
   }

   pEhResTrk->isInitialized = false; /* we're done */

   FUN_LEAVE();
} /* ehResTerm() */


/***
ehResTreeDump()
print out the members of the resource TREE structure.
***/

void
ehResTreeDump A1(EHAVL*, pEhAvl)
{
   fprintf(EHSTDERR, "------- RESLIST/RESCALL dump -------\n");
   fflush(EHSTDERR);

   if (pEhAvl->pEhAvlNodeTop) resDump(pEhAvl->pEhAvlNodeTop, 0);

   fprintf(EHSTDERR, "------------------------------------\n");
   fflush(EHSTDERR);
} /* ehDmpResTree */


/***
resDump()
print out the members of the resource TREE structure.
***/

static
void
resDump A2(EHAVLNODE*, pEhAvlNode,
                int,   iLevel)
{
   if (pEhAvlNode->pEhAvlNodeUp)
      resDump(pEhAvlNode->pEhAvlNodeUp, iLevel+1);

   resDumpNode(pEhAvlNode->pvUserData, iLevel+1);

   if (pEhAvlNode->pEhAvlNodeDn)
      resDump(pEhAvlNode->pEhAvlNodeDn, iLevel+1);

} /* resDump */


/***
resDumpNode()
This function prints out the contents
of an AvlNode and the corresponding user data.
***/

static
void
resDumpNode A2(
   const void *,    pvRes,
   int,       iLevel)
{
   EHRESLIST *pRes = (EHRESLIST *)pvRes;
   EHRESCALL *pEhResCall;
   size_t   sz;

   if (iLevel == 1)
      fprintf(EHSTDERR, "==>");
   else
      fprintf(EHSTDERR, "%*.*s", iLevel*3, iLevel*3, "");

   fprintf(EHSTDERR, "ulResHndl: 0x%lx, ", pRes->ulResHndl);
   fprintf(EHSTDERR, "iType: %d", pRes->iType);
   if (pRes->pcResNam)
   {
      fprintf(EHSTDERR, ", pcResNam: %s", pRes->pcResNam);
   }
   fprintf(EHSTDERR, "\n");

   fprintf(EHSTDERR, "%*.*s", iLevel*3, iLevel*3, "");
   fprintf(EHSTDERR, "EhResState: %d, ", pRes->EhResState);
   fprintf(EHSTDERR, "EhResWhom: %d\n", pRes->EhResWhom);

   fprintf(EHSTDERR, "%*.*s", iLevel*3, iLevel*3, "");
   fprintf(EHSTDERR, "szRcAlloc: %d, ", pRes->szRCAlloc);
   fprintf(EHSTDERR, "szRcInUse: %d, ", pRes->szRCInUse);
   fprintf(EHSTDERR, "uhNumObtn: %u, ", pRes->uhNumObtn);
   fprintf(EHSTDERR, "pEhResCall:  0x%p\n", pRes->pEhResCall);

   pEhResCall = pRes->pEhResCall;
   for (sz = pRes->szRCAlloc; sz--; ++pEhResCall)
   {
      fprintf(EHSTDERR, "%*.*s", iLevel*3, iLevel*3, "");

      if (pEhResCall->pulArgs)
     fprintf(EHSTDERR, "pulArgs: 0x%p", pEhResCall->pulArgs);

      if (pEhResCall->pcFileO)
      {
         if (pEhResCall->pulArgs) fprintf(EHSTDERR, ", ");
         fprintf(EHSTDERR, "O<%s %s%s%d>",
               pEhResCall->pcFileO,
               pEhResCall->pcFuncO?pEhResCall->pcFuncO:"",
               pEhResCall->pcFuncO?"() ":"",
               pEhResCall->iLineO);
      }

      if (pEhResCall->pcFileR)
      {
         if (pEhResCall->pcFileO) fprintf(EHSTDERR, ", ");
         fprintf(EHSTDERR, "R<%s %s%s%d>",
               pEhResCall->pcFileR,
               pEhResCall->pcFuncR ? pEhResCall->pcFuncR : "",
               pEhResCall->pcFuncR ? "() " : "",
               pEhResCall->iLineR);
      }
      fprintf(EHSTDERR, "\n");
   }
   fflush(EHSTDERR);
} /* resDumpNode */


static
void
resDumpUserData A2(FILE *, pFile,
                   const void *, pvUserData)
{
   EHRESLIST *pRes;
   EHRESCALL *pEhResCall;
   size_t   sz;

   pRes = (EHRESLIST *)pvUserData; /* user data */

   fprintf(pFile, "H:0x%lx,", pRes->ulResHndl);
   fprintf(pFile, "T:%d", pRes->iType);
   if (pRes->pcResNam)
   {
      fprintf(pFile, ",N:%s", pRes->pcResNam);
   }

   fprintf(pFile, ",S:%d", pRes->EhResState);
   fprintf(pFile, ",W:%d", pRes->EhResWhom);

   fprintf(pFile, ",A:%d", pRes->szRCAlloc);
   fprintf(pFile, ",U:%d", pRes->szRCInUse);
   fprintf(pFile, ",O:%u", pRes->uhNumObtn);
   fprintf(pFile, ",C:0x%p", pRes->pEhResCall);
   fflush(pFile);

   pEhResCall = pRes->pEhResCall;
   for (sz = pRes->szRCAlloc; sz--; ++pEhResCall)
   {

      if (pEhResCall->pulArgs)
      {
         //puts("resDumpUserData: pEhResCall->pulArgs");
         fprintf(pFile, ",A:0x%p", pEhResCall->pulArgs);
      }

      if (pEhResCall->pcFileO)
      {
         if (pEhResCall->pulArgs)
         {
            fprintf(pFile, ",");
         }
         //puts("\nresDumpUserData: pEhResCall->pcFileO,pcFuncO,iLineO");
         fprintf(pFile, "O<%s %s%s%d>",
               pEhResCall->pcFileO,
               pEhResCall->pcFuncO?pEhResCall->pcFuncO:"",
               pEhResCall->pcFuncO?"() ":"",
               pEhResCall->iLineO);
      }

      if (pEhResCall->pcFileR)
      {
         if (pEhResCall->pcFileO) fprintf(pFile, ",");
         //puts("\nresDumpUserData: pEhResCall->pcFileR,pcFuncR,iLineR");
         fprintf(pFile, "R<%s %s%s%d>",
               pEhResCall->pcFileR,
               pEhResCall->pcFuncR?pEhResCall->pcFuncR:"",
               pEhResCall->pcFuncR?"() ":"",
               pEhResCall->iLineR);
      }
   }
   fflush(pFile);
} /* resDumpUserData */


/***
cmpFunList()
This function compares two EHRESFUNC elements.  It returns zero if the
iLine/pcFunc members of the two elements are equal, -1 if the first arg
is less than the second, and +1 if the first is greater than the second.

RETURN VALUES:
-1: first < second
 0: first == second
+1: first > second
***/

static 
int 
cmpFunList A2(const void *, pv1,
              const void *, pv2)
{
   int iLine1    = ((EHRESFUNC*)pv1)->iLine;
   int iLine2    = ((EHRESFUNC*)pv2)->iLine;
   volatile int iRetVal = 0;

   EHCHAN(2);
   FUN_TRACE("cmpFunList");   /* track entry into this function */
   
#if (EH_CHAN == 0) && 0

   EHDBG2("iLine1 = %d, iLine2 = %d",iLine1,iLine2);

   if (iLine1 < iLine2)
   {
      iRetVal = -1;
   }
   else if (iLine1 > iLine2)
   {
      iRetVal = 1;
   }
   else
   {
      char *pcFile1 = ((EHRESFUNC*)pv1)->pcFile;
      char *pcFile2 = ((EHRESFUNC*)pv2)->pcFile;

      iRetVal = strcmp(pcFile1, pcFile2);
   }

#else

   iRetVal = (iLine1 - iLine2);
   if (iRetVal == 0)
   {
      const char *pcFile1 = ((EHRESFUNC*)pv1)->pcFile;
      const char *pcFile2 = ((EHRESFUNC*)pv2)->pcFile;

      iRetVal = strcmp(pcFile1, pcFile2);
   }

#endif

   EHDBG1("returning %i", iRetVal);
   FUN_RETURN(iRetVal);
} /* cmpFunList */


/***
freeResFunc()
Free all components of a RESFUNC structure
***/

static
int
freeResFunc A1(const void**, ppv)
{
   EHRESFUNC**ppEhResFunc = (EHRESFUNC**)ppv;

   FUN_TRACE("freeResFunc");

   if (*ppEhResFunc)
      EHFREE(*ppEhResFunc);

   FUN_RETURN(0);
} /* freeResFunc */


static
void
funDumpUserData A2(FILE *, pFile,
                   const void *, pvUserData)
{
   EHRESFUNC *pResFunc;

   pResFunc = (EHRESFUNC *)pvUserData; /* user data */

   fprintf(pFile, "L:%d,", pResFunc->iLine);
   fprintf(pFile, "F:%s",  pResFunc->pcFile);
   fflush(pFile);
} /* funDumpUserData */



