/***

ehvolume.c -- Error Handler, set channel/volume values

Copyright (c) 1992-6 Christopher Cobb, All rights reserved.

***/

static const char *rcsid =
       "$Id: ehvolume.c,v 1.3 1995/08/16 19:49:03 ccobb Exp $";

#include "ehenv.h"

EHBEGINEXTERNC
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
EHENDEXTERNC

#include "ehchnvol.h"
#include "ehvolume.h"

#if !defined(tolower)
int  tolower A((int c));
#endif

#if !defined(toupper)
int  toupper A((int c));
#endif

/***
ehVolume() is called early during
initialization and therefore is
not an Error Handler compliant
function.
***/

int
ehVolume A1(const char *, pcEhVolume)
{
   int iRetVal = 0;
   int iChan =  0; /* by default, operate on channel 0 */
   int  iVol = -1;
   int  iFun = -1;
   int  iMem = -1;
   int  iOut = -1;
   char *pc;
   char *pcMem;
   char *pcVolume;
   char *pcSetting;

   /* make temp working buffer for environment variable */
   if ((pc = pcMem = (char *)calloc(strlen(pcEhVolume) + 1, (size_t)1)) == NULL)
   {
      fprintf(EHSTDERR, "calloc() failed\n");
      fflush(EHSTDERR);
      iRetVal = -3;
      goto Exit;
   }
   pcVolume = strcpy(pc, pcEhVolume);

   /* chop out sections */
   while ((pcSetting = strtok(pcVolume, "- \"")) != NULL)
   {

      pcVolume = NULL;   /* for strtok() */

      // printf("ehVolume: calling EhSetVal(%s,...)\n", pcSetting);
      if (ehSetVal(pcSetting, &iChan, &iVol, &iFun, &iMem, &iOut))
      {
         fprintf(EHSTDERR, "bad EHVOLUME option: %s\n", pcSetting);
         fflush(EHSTDERR);
         continue;
      }
   } /* end while */

Exit:
   if (pcMem)
      free (pcMem);
   return iRetVal;
}


int
ehSetVal A6(char *,  pcSetting,
             int *,  piChan,
             int *,  piVol,
             int *,  piFun,
             int *,  piMem,
             int *,  piOut)
{
   char cChan;
   char cVol;
   int  iRetVal = 0;

   // puts("ehSetVal: entering");

   if (*pcSetting == '-')
   {
      ++pcSetting;
   }
   switch (*pcSetting)
   {
   case 'c' : /*** check for channel ***/
      cChan = *(pcSetting + 1);
      if (isdigit((int)cChan))
      {
         *piChan = cChan - '0';
      }
      else if (isalpha((int)cChan))
      {
         if (isupper((int)cChan))
            cChan = (char)tolower((int)cChan);
         for (*piChan = SZARR(apcEhChan); (*piChan)--; )
         {
            if (*apcEhChan[*piChan] == cChan)
            {
              break;
            }
         }
      }
      if (cChan != '@' && (*piChan < 0 || *piChan >= SZARR(apcEhChan)))
      {
         fprintf(EHSTDERR, "invalid channel: %s\n", pcSetting);
         fflush(EHSTDERR);
         iRetVal = 1;
         goto Exit;
      }
      break;

   case 'v' : /*** check for volume ***/
      cVol = (char)toupper((int)*(pcSetting + 1));
      if (isdigit((int)cVol))
      {
         *piVol = cVol - '0';
      }
      else
      {
         for (*piVol = SZARR(apcEhVol); (*piVol)--; )
         {
            if (*apcEhVol[*piVol] == cVol)          
            {
               break;
            }
         }
      }
      if (*piVol < 0 || *piVol >= SZARR(apcEhVol))
      {
         fprintf(EHSTDERR, "invalid volume: %s\n", pcSetting);
         fflush(EHSTDERR);
         iRetVal = 2;
         goto Exit;
      }
      /* set the volume on the specified channel(s) */
      if (*piChan < 0)
      {
         for (*piChan = SZARR(apcEhChan); (*piChan)--; )
         {
            ehSetVol(*piChan, *piVol);
         }
      }
      else
      {
         ehSetVol(*piChan, *piVol);
      }
      break;

   case 'f' : /*** check for function tracing ***/
      *piFun = atoi(pcSetting + 1);
      /* set function tracing on the specified channel(s) */
      if (*piChan < 0)
      {
         for (*piChan = SZARR(apcEhChan); (*piChan)--; )
         {
            ehSetFun(*piChan, *piFun);
         }
      }
      else
      {
         ehSetFun(*piChan, *piFun);
      }
      break;

   case 'm' : /*** check for memory tracing ***/
      *piMem = atoi(pcSetting + 1);
      /* set memory tracing on the specified channel(s) */
      if (*piChan < 0)
      {
         for (*piChan = SZARR(apcEhChan); (*piChan)--; )
         {
            ehSetMem(*piChan, *piMem);
         }
      }
      else
      {
         ehSetMem(*piChan, *piMem);
      }
      break;

   case 'o' : /*** select output file ***/
      *piOut = atoi(pcSetting + 1);
      /* set output file setting */
      ehSetOut(*piOut);
      break;

   default:
      fprintf(EHSTDERR, "Unrecognized setting: %s\n", pcSetting);
      fflush(EHSTDERR);
      break;

   }
Exit:
   // puts("ehSetVal: leaving");

   return (iRetVal);
}



