/***

ehwldcrd.c -- Error Handler, filename wild card Support

Copyright (c) 1992-6 Christopher Cobb, All rights reserved.

***/

static const char*rcsid =
       "$Id: ehwldcrd.c,v 1.2 1995/12/04 21:46:38 ccobb Exp $";

/*** system headers ***/

#include "ehenv.h"

EHBEGINEXTERNC

#include <string.h>

#if ehAMIGA

#if ehSASC
#include <pragmas/dos_pragmas.h>
#endif

#if ehMX
#include <utility/tagitem.h> /* struct TagItem */
#include <dos/exall.h>       /* struct ExAllData */
#include <pragmas/dos_lib.h>
#endif

#if ehDCC
#endif

#if ehGNUC
#include <utility/tagitem.h> /* struct TagItem */
#include <dos/exall.h>       /* struct ExAllData */
#endif

#include <dos/dos.h>
#include <clib/dos_protos.h>

extern struct Library *DOSBase;

#endif /* ehAMIGA */

EHENDEXTERNC

/*** portability headers ***/
#include "eh.h"
#include "eherrs.h"
#include "ehdynarr.h"

FILENAME("ehwldcrd.c");

/*** project headers ***/

/*** local (subsystem) headers ***/
#include "ehwldcrd.h"

/* make these static so we can free them later */
static size_t szArgcSav;
static char **ppcArgvSav;
static size_t szAllocated;
static char  *pcAllocated;

static short hasWildCards   A((const char *));
static void  addMatches     A((char*pcArgv));
static void  freeArgvMemory A((void));

/***

This function receives pointers to argc and argv.
It scans argv to see if there are any wild cards.
If there are none, we're done.  If there are,
argv is re-allocated and re-populated to resolve
any wild cards and replace the argv entries with
unambigous file names.

***/

void
ehExpWildCards A2(int *, piArgc, 
                  char ***, pppcArgv)
{
   int iArgc;
   char **ppcArgv;
   char **ppcNewArg;
   char *pcMark;

   EHTRACE(ehExpWildCards);

#if ehNIX
   EHLEAVE(); /* no expansion is necessary */
#endif

   /*** are there any wildcards? ***/
   for (iArgc = *piArgc, ppcArgv = *pppcArgv; iArgc--; ++ppcArgv)
   {
      /*** does ppcArgv have any wild cards? ***/
      if (hasWildCards(*ppcArgv))
         break;
   }
   if (iArgc < 0) /* loop was exhausted with no wildcards found */
      EHLEAVE(); /* no expansion is necessary */

   /*** if there are, we need to re-allocate argv ***/

   for (iArgc = 0, ppcArgv = *pppcArgv; iArgc < *piArgc; ++iArgc, ++ppcArgv)
   {
      if (hasWildCards(*ppcArgv))
      {
         /* add zero or more entries to ppvArgv */
         addMatches(*ppcArgv);
      }
      else /* no wildcards */
      {
         /* just copy entry over */
         ppcNewArg = (char**)ehDynArr(
               (void**)&ppcArgvSav, &szArgcSav, sizeof(char*));
         if (ppcNewArg == NULL) EHTHROW_LEAVE(EH_DYNARR);
         *ppcNewArg = *ppcArgv;

         /* mark as not allocated */
         pcMark = (char*)ehDynArr(
               (void**)&pcAllocated, &szAllocated, sizeof(char));
         if (pcMark == NULL) EHTHROW_LEAVE(EH_DYNARR);
         *pcMark = '\0';
      }
   }

   /* null-terminate the list */
   ppcNewArg = (char**)ehDynArr((void**)&ppcArgvSav, &szArgcSav, sizeof(char*));
   if (ppcNewArg == NULL) EHTHROW_LEAVE(EH_DYNARR);
   *ppcNewArg = NULL;

   /* mark as not allocated */
   pcMark = (char*)ehDynArr((void**)&pcAllocated, &szAllocated, sizeof(char));
   if (pcMark == NULL) EHTHROW_LEAVE(EH_DYNARR);
   *pcMark = '\0';

   if (ppcArgvSav)
   {
      *piArgc = (int)szArgcSav;
      *pppcArgv = ppcArgvSav;
      ehAtExit(freeArgvMemory);
   }
   EHLEAVE();
}

static
short
hasWildCards A1(const char *, pcArgv)
#ifdef HAVE_CONFIG_H
{pcArgv=pcArgv;return 0;}
#endif
#if ehAMIGA
{
   char *volatile pcDest = NULL;
   volatile LONG lIsWild = -1;
   LONG lLen;

   EHCATCH(hasWildCards) {

      /* set up destination buffer */
      lLen = 3 * strlen(pcArgv) + 2;
      pcDest = (char*)calloc((int)lLen, 1);
      if (pcDest == NULL) EHTHROW_EXITSEQ(EH_CALLOC);

      lIsWild = ParsePatternNoCase((STRPTR)pcArgv, (STRPTR)pcDest, lLen);
      if (lIsWild < 0) EHTHROW_EXITSEQ(EH_PARSEPATTERNOCASE);

   } EHENDCATCH;

   if (pcDest) free(pcDest);
   EHRETURN((short)(lIsWild == 1));
}
#endif /* ehAMIGA */


static
void
addMatches A1(char*,    pcArgv)
#ifdef HAVE_CONFIG_H
{pcArgv=pcArgv;}
#endif
#if ehAMIGA
{
   struct AnchorPath * volatile pAnchorPath = NULL;
   char **ppcNewArgv;
   LONG lError = 0;
   char *pcMark;

   EHCATCH(addMatches) {

      pAnchorPath = NEW(struct AnchorPath);
      if (pAnchorPath == NULL) EHTHROW_EXITSEQ(EH_NEWMACRO);

      lError = MatchFirst((STRPTR)pcArgv, pAnchorPath);
      while (lError == 0)
      {
         EHWRN1("found: [%s]", pAnchorPath->ap_Info.fib_FileName);

         /* add slot in argv for file name */
         ppcNewArgv = (char**)ehDynArr((void**)&ppcArgvSav, &szArgcSav, sizeof(char*));
         if (ppcNewArgv == NULL) EHTHROW_EXITSEQ(EH_DYNARR);

         /* duplicate the string */
         *ppcNewArgv = (char*)calloc(
               strlen(pAnchorPath->ap_Info.fib_FileName)+1,1);
         if (*ppcNewArgv == NULL) EHTHROW_EXITSEQ(EH_CALLOC);
         strcpy(*ppcNewArgv, pAnchorPath->ap_Info.fib_FileName);

         /* mark as allocated */
         pcMark = (char*)ehDynArr(
               (void**)&pcAllocated, &szAllocated, sizeof(char));
         if (pcMark == NULL) EHTHROW_LEAVE(EH_DYNARR);
         *pcMark = '\1';

         lError = MatchNext(pAnchorPath);
      }
      if (lError != ERROR_NO_MORE_ENTRIES)
      {
         ERR1("lError = %d", lError);
         EHTHROW_EXITSEQ(EH_MATCH);
      }

   } EHENDCATCH;

   if (pAnchorPath)
   {
      MatchEnd(pAnchorPath);
      free(pAnchorPath);
   }
   EHLEAVE();
}
#endif /* ehAMIGA */


static
void
freeArgvMemory A0()
{
   char  *pc;
   char **ppc;

   EHCATCH(freeMemory) {

      EHASSERT_LEAVE(szArgcSav == szAllocated);

   } EHENDCATCH;

   pc  = pcAllocated + szAllocated;
   ppc = ppcArgvSav  + szArgcSav;

   while (szAllocated--)
   {
      EHWRN1("szAllocated = [%d]", szAllocated);
      --ppc;
      if (*--pc == '\1')
         free(*ppc);
   }
   if (pcAllocated) free(pcAllocated);
   if (ppcArgvSav)  free(ppcArgvSav);

   EHLEAVE();
}


