/***

t_pgm.c -- Error Handler, simple test for Error Handler

Copyright (c) 1992-6 Christopher Cobb, All rights reserved.

***/

static const char*rcsid =
       "$Id: t_pgm.c,v 1.5 1996/03/04 06:46:59 ccobb Exp $";

#define _POSIX_SOURCE 1

#include "eh.h"

EHBEGINEXTERNC
#include <stdio.h>
#include <string.h>
EHENDEXTERNC

#include "ehercode.h"

EHFILENAME(t_pgm.c);

static int  cmpER(ER*pEr1,ER*pEr2);

int 
main A2(
   int,    argc, 
   char**, argv)
{
   ER Er1;
   ER Er2;
   ulong ulHash;

   EHMAIN(argc, argv);

   Er1.hCode = 2;

   strcpy(Er1.acPrefix, "");
   if (ehHash(Er1) != ~0)
      puts("------- Error: allowed an empty prefix string");

   memcpy(Er1.acPrefix, "a2345", 5);
   if (ehHash(Er1) != ~0)
      puts("------- Error: allowed a five-char prefix string");

   strcpy(Er1.acPrefix, "a23_");
   if (ehHash(Er1) != ~0)
      puts("------- Error: allowed a non-alpha-numeric in prefix string");

   strcpy(Er1.acPrefix, "a2_");
   if (ehHash(Er1) != ~0)
      puts("------- Error: allowed a non-alpha-numeric in prefix string");

   strcpy(Er1.acPrefix, "a_");
   if (ehHash(Er1) != ~0)
      puts("------- Error: allowed a non-alpha-numeric in prefix string");

   strcpy(Er1.acPrefix, "_");
   if (ehHash(Er1) != ~0)
      puts("------- Error: allowed a non-alpha-numeric in prefix string");

   strcpy(Er1.acPrefix, "0");
   if (ehHash(Er1) != ~0)
      puts("------- Error: allowed a numeric in first char of prefix string");

   Er1.hCode = 1;
   strcpy(Er1.acPrefix, "a");
   if (ehHash(Er1) != ~0)
      puts("------- Error: allowed an error code less than 2");

   Er1.hCode = 3262;
   strcpy(Er1.acPrefix, "z");
   if (ehHash(Er1) != ~0)
      puts("------- Error: allowed an error code greater than 3261");

   /*** a-2 ***/

   Er1.hCode = 2;
   memcpy(Er1.acPrefix, "a\0\0\0\0", EH_SZ_PREFIX);
   if ((ulHash = ehHash(Er1)) == ~0)
      puts("------- Error: should have allowed error a-2");

   Er2 = ehUnHash(ulHash);
   if (cmpER(&Er1, &Er2) != 0 )
      puts("------- Error: hashed a-2 != unhashed a-2");

   /*** z-3261 ***/

   Er1.hCode = 3261;
   memcpy(Er1.acPrefix, "z\0\0\0\0", EH_SZ_PREFIX);
   if ((ulHash = ehHash(Er1)) == ~0)
      puts("------- Error: should have allowed error z-3261");

   Er2 = ehUnHash(ulHash);
   if (cmpER(&Er1, &Er2) != 0 )
      puts("------- Error: hashed z-3261 != unhashed z-3261");

   /*** aa-11 ***/

   Er1.hCode = 11;
   memcpy(Er1.acPrefix, "aa\0\0\0", EH_SZ_PREFIX);
   if ((ulHash = ehHash(Er1)) == ~0)
      puts("------- Error: should have allowed error aa-11");

   Er2 = ehUnHash(ulHash);
   if (cmpER(&Er1, &Er2) != 0 )
      puts("------- Error: hashed aa-11 != unhashed aa-11");

   /*** az-19 ***/

   Er1.hCode = 19;
   memcpy(Er1.acPrefix, "az\0\0\0", EH_SZ_PREFIX);
   if ((ulHash = ehHash(Er1)) == ~0)
      puts("------- Error: should have allowed error az-19");

   Er2 = ehUnHash(ulHash);
   if (cmpER(&Er1, &Er2) != 0 )
      puts("------- Error: hashed az-19 != unhashed az-19");

   /*** a0-10 ***/

   Er1.hCode = 10;
   memcpy(Er1.acPrefix, "a0\0\0\0", EH_SZ_PREFIX);
   if ((ulHash = ehHash(Er1)) == ~0)
      puts("------- Error: should have allowed error a0-10");

   Er2 = ehUnHash(ulHash);
   if (cmpER(&Er1, &Er2) != 0 )
      puts("------- Error: hashed a0-10 != unhashed a0-10");

   /*** a9-99 ***/

   Er1.hCode = 99;
   memcpy(Er1.acPrefix, "a9\0\0\0", EH_SZ_PREFIX);
   if ((ulHash = ehHash(Er1)) == ~0)
      puts("------- Error: should have allowed error a9-99");

   Er2 = ehUnHash(ulHash);
   if (cmpER(&Er1, &Er2) != 0 )
      puts("------- Error: hashed a9-99 != unhashed a9-99");

   /*** a#-90 ***/

   Er1.hCode = 90;
   memcpy(Er1.acPrefix, "a#\0\0\0", EH_SZ_PREFIX);
   if ((ulHash = ehHash(Er1)) != ~0)
      puts("------- Error: should not have allowed error a#-90");

   /*** aaa-111 ***/

   Er1.hCode = 111;
   memcpy(Er1.acPrefix, "aaa\0\0\0", EH_SZ_PREFIX);
   if ((ulHash = ehHash(Er1)) == ~0)
      puts("------- Error: should have allowed error aaa-111");

   Er2 = ehUnHash(ulHash);
   if (cmpER(&Er1, &Er2) != 0 )
      puts("------- Error: hashed aaa-111 != unhashed aaa-111");

   /*** zaz-919 ***/

   Er1.hCode = 919;
   memcpy(Er1.acPrefix, "zaz\0\0\0", EH_SZ_PREFIX);
   if ((ulHash = ehHash(Er1)) == ~0)
      puts("------- Error: should have allowed error zaz-919");

   Er2 = ehUnHash(ulHash);
   if (cmpER(&Er1, &Er2) != 0 )
      puts("------- Error: hashed zaz-919 != unhashed zaz-919");

   /*** aa0-110 ***/

   Er1.hCode = 110;
   memcpy(Er1.acPrefix, "aa0\0\0\0", EH_SZ_PREFIX);
   if ((ulHash = ehHash(Er1)) == ~0)
      puts("------- Error: should have allowed error aa0-110");

   Er2 = ehUnHash(ulHash);
   if (cmpER(&Er1, &Er2) != 0 )
      puts("------- Error: hashed aa0-110 != unhashed aa0-110");

   /*** za9-909 ***/

   Er1.hCode = 909;
   memcpy(Er1.acPrefix, "za9\0\0\0", EH_SZ_PREFIX);
   if ((ulHash = ehHash(Er1)) == ~0)
      puts("------- Error: should have allowed error za9-909");

   Er2 = ehUnHash(ulHash);
   if (cmpER(&Er1, &Er2) != 0 )
      puts("------- Error: hashed za9-909 != unhashed za9-909");

   /*** za#-990 ***/

   Er1.hCode = 990;
   memcpy(Er1.acPrefix, "za#\0\0\0", EH_SZ_PREFIX);
   if ((ulHash = ehHash(Er1)) != ~0)
      puts("------- Error: should not have allowed error za#-990");

   /*** aaaa-1111 ***/

   Er1.hCode = 1111;
   memcpy(Er1.acPrefix, "aaaa\0\0\0", EH_SZ_PREFIX);
   if ((ulHash = ehHash(Er1)) == ~0)
      puts("------- Error: should have allowed error aaa-1111");

   Er2 = ehUnHash(ulHash);
   if (cmpER(&Er1, &Er2) != 0 )
   {
      puts("------- Error: hashed aaaa-1111 != unhashed aaaa-1111");
      dmpER(&Er1);
      dmpER(&Er2);
   }

   /*** azaz-1919 ***/

   Er1.hCode = 1919;
   memcpy(Er1.acPrefix, "zaz\0\0\0", EH_SZ_PREFIX);
   if ((ulHash = ehHash(Er1)) == ~0)
      puts("------- Error: should have allowed error azaz-1919");

   Er2 = ehUnHash(ulHash);
   if (cmpER(&Er1, &Er2) != 0 )
      puts("------- Error: hashed azaz-1919 != unhashed azaz-1919");

   /*** aa09-1109 ***/

   Er1.hCode = 1109;
   memcpy(Er1.acPrefix, "aa09\0\0\0", EH_SZ_PREFIX);
   if ((ulHash = ehHash(Er1)) == ~0)
      puts("------- Error: should have allowed error aa09-1109");

   Er2 = ehUnHash(ulHash);
   if (cmpER(&Er1, &Er2) != 0 )
      puts("------- Error: hashed aa09-1109 != unhashed aa09-1109");

   /*** za9z-9099 ***/

   Er1.hCode = 9099;
   memcpy(Er1.acPrefix, "za9z\0\0\0", EH_SZ_PREFIX);
   if ((ulHash = ehHash(Er1)) != ~0)
      puts("------- Error: should not have allowed error za9z-9099");

   /*** za#z-9909 ***/

   Er1.hCode = 9909;
   memcpy(Er1.acPrefix, "za#z\0\0\0", EH_SZ_PREFIX);
   if ((ulHash = ehHash(Er1)) != ~0)
      puts("------- Error: should not have allowed error za#z-9909");

   /*** highest hash code ***/

   Er1.hCode = 3260;
   memcpy(Er1.acPrefix, "z999\0\0\0\0", EH_SZ_PREFIX);
   if ((ulHash = ehHash(Er1)) == ~0)
      puts("------- Error: should have allowed error z999-3260");
   else
   {
      printf("ulHash(z999-3260) = 0x%lx\n", ulHash);
      printf("0x%lx unhashes to:\n", ulHash);
      Er2 = ehUnHash(ulHash);
      dmpER(&Er2);
   }

   Er1.hCode = 3261;
   memcpy(Er1.acPrefix, "z999\0\0\0\0", EH_SZ_PREFIX);
   if ((ulHash = ehHash(Er1)) == ~0)
      puts("------- Error: should have allowed error z999-3261");
   else
   {
      printf("ulHash(z999-3261) = 0x%lx\n", ulHash);
      printf("0x%lx unhashes to:\n", ulHash);
      Er2 = ehUnHash(ulHash);
      dmpER(&Er2);
   }

   Er1.hCode = 3262;
   memcpy(Er1.acPrefix, "z999\0\0\0\0", EH_SZ_PREFIX);
   if ((ulHash = ehHash(Er1)) == ~0)
      puts("------- Error: should have allowed error z999-3262");
   else
   {
      printf("ulHash(z999-3262) = 0x%lx\n", ulHash);
      printf("0x%lx unhashes to:\n", ulHash);
      Er2 = ehUnHash(ulHash);
      dmpER(&Er2);
   }

   /*** special info ***/

   Er1.hCode = 1;
   memcpy(Er1.acPrefix, "eh\0\0\0\0", EH_SZ_PREFIX);
   if ((ulHash = ehHash(Er1)) == ~0)
      puts("------- Error: should have allowed error a-2");
   else
      printf("ulHash(eh-1) = 0x%lx\n", ulHash);

   Er2 = ehUnHash(EH_CTRLC_HASH);
   puts("EH_CTRLC_HASH unhashes to:");
   dmpER(&Er2);

ExitSequence:
   EHENDMAIN(ehError());
}


static
int
cmpER A2(
   ER*, pEr1,
   ER*, pEr2)
{
   return (pEr1->hCode - pEr2->hCode) ||
           strncmp(pEr1->acPrefix, pEr2->acPrefix, EH_SZ_PREFIX);
}




